/* Copyright (c) 2001 Matej Pfajfar.
 * Copyright (c) 2001-2004, Roger Dingledine.
 * Copyright (c) 2004-2006, Roger Dingledine, Nick Mathewson.
 * Copyright (c) 2007-2021, The Tor Project, Inc. */
/* See LICENSE for licensing information */

/**
 * \file risky_options.h
 * \brief Header for risky_options.c
 **/

#ifndef TOR_RISKY_OPTIONS_H
#define TOR_RISKY_OPTIONS_H

extern const char risky_option_list[];

#endif /* !defined(TOR_RISKY_OPTIONS_H) */
