/* Copyright (c) 2001 Matej Pfajfar.
 * Copyright (c) 2001-2004, Roger Dingledine.
 * Copyright (c) 2004-2006, Roger Dingledine, Nick Mathewson.
 * Copyright (c) 2007-2021, The Tor Project, Inc. */
/* See LICENSE for licensing information */

/**
 * @file evloop_sys.h
 * @brief Declare subsystem object for the event loop module.
 **/

#ifndef TOR_LIB_EVLOOP_EVLOOP_SYS_H
#define TOR_LIB_EVLOOP_EVLOOP_SYS_H

extern const struct subsys_fns_t sys_evloop;

#endif /* !defined(TOR_LIB_EVLOOP_EVLOOP_SYS_H) */
