/* Copyright (c) 2021-2021, The Tor Project, Inc. */
/* See LICENSE for licensing information */

/**
 * @file dos_config.h
 * @brief Header for core/or/dos_config.c
 **/

#ifndef TOR_CORE_OR_DOS_CONFIG_H
#define TOR_CORE_OR_DOS_CONFIG_H

#include "lib/conf/conftypes.h"

extern const struct config_format_t dos_options_fmt;

#endif /* !defined(TOR_CORE_OR_DOS_CONFIG_H) */
