/* Copyright (c) 2003-2004, Roger Dingledine
 * Copyright (c) 2004-2006, Roger Dingledine, Nick Mathewson.
 * Copyright (c) 2007-2021, The Tor Project, Inc. */
/* See LICENSE for licensing information */

/**
 * \file numcpus.h
 * \brief Header for numcpus.c
 **/

#ifndef TOR_NUMCPUS_H
#define TOR_NUMCPUS_H

int compute_num_cpus(void);

#endif /* !defined(TOR_NUMCPUS_H) */
