/* Copyright (c) 2001 Matej Pfajfar.
 * Copyright (c) 2001-2004, Roger Dingledine.
 * Copyright (c) 2004-2006, Roger Dingledine, Nick Mathewson.
 * Copyright (c) 2007-2021, The Tor Project, Inc. */
/* See LICENSE for licensing information */

/**
 * \file proxymode.h
 * \brief Header file for proxymode.c.
 **/

#ifndef TOR_PROXYMODE_H
#define TOR_PROXYMODE_H

int proxy_mode(const or_options_t *options);

#endif /* !defined(TOR_PROXYMODE_H) */
