/* Copyright (c) 2018-2021, The Tor Project, Inc. */
/* See LICENSE for licensing information */

/**
 * \file wallclock_sys.h
 * \brief Declare subsystem object for the wallclock module.
 **/

#ifndef TOR_WALLCLOCK_SYS_H
#define TOR_WALLCLOCK_SYS_H

extern const struct subsys_fns_t sys_wallclock;

#endif /* !defined(TOR_WALLCLOCK_SYS_H) */
