/* Copyright 2001-2004 Roger Dingledine.
 * Copyright (c) 2004-2006, Roger Dingledine, Nick Mathewson.
 * Copyright (c) 2007-2021, The Tor Project, Inc. */
/* See LICENSE for licensing information */

#ifndef TOR_GIT_REVISION_H
#define TOR_GIT_REVISION_H

/**
 * @file git_revision.h
 * @brief Header for git_revision.c
 **/

extern const char tor_git_revision[];
extern const char tor_bug_suffix[];

#endif /* !defined(TOR_GIT_REVISION_H) */
