/* Copyright (c) 2007-2021, The Tor Project, Inc. */
/* See LICENSE for licensing information */

/**
 * \file btrack_sys.h
 * \brief Declare subsystem object for the bootstrap tracker susbystem.
 **/

#ifndef TOR_BTRACK_SYS_H
#define TOR_BTRACK_SYS_H

extern const struct subsys_fns_t sys_btrack;

#endif /* !defined(TOR_BTRACK_SYS_H) */
