/* Copyright (c) 2018-2021, The Tor Project, Inc. */
/* See LICENSE for licensing information */

/**
 * \file winprocess_sys.h
 * \brief Declare subsystem object for winprocess.c
 **/

#ifndef TOR_WINPROCESS_SYS_H
#define TOR_WINPROCESS_SYS_H

extern const struct subsys_fns_t sys_winprocess;

#endif /* !defined(TOR_WINPROCESS_SYS_H) */
