/* Copyright (c) 2001 Matej Pfajfar.
 * Copyright (c) 2001-2004, Roger Dingledine.
 * Copyright (c) 2004-2006, Roger Dingledine, Nick Mathewson.
 * Copyright (c) 2007-2021, The Tor Project, Inc. */
/* See LICENSE for licensing information */

/**
 * @file mainloop_sys.h
 * @brief Header for mainloop_sys.c
 **/

#ifndef MAINLOOP_SYS_H
#define MAINLOOP_SYS_H

extern const struct subsys_fns_t sys_mainloop;

#endif /* !defined(MAINLOOP_SYS_H) */
