/* Copyright (c) 2018-2021, The Tor Project, Inc. */
/* See LICENSE for licensing information */

/**
 * \file network_sys.h
 * \brief Declare subsystem object for the network module.
 **/

#ifndef TOR_NETWORK_SYS_H
#define TOR_NETWORK_SYS_H

extern const struct subsys_fns_t sys_network;

#endif /* !defined(TOR_NETWORK_SYS_H) */
