/* Copyright 2001-2004 Roger Dingledine.
 * Copyright (c) 2004-2006, Roger Dingledine, Nick Mathewson.
 * Copyright (c) 2007-2021, The Tor Project, Inc. */
/* See LICENSE for licensing information */

#ifndef TOR_VERSION_H
#define TOR_VERSION_H

/**
 * @file torversion.h
 * @brief Header for version.c.
 **/

const char *get_version(void);
const char *get_short_version(void);

#endif /* !defined(TOR_VERSION_H) */
