/* Copyright (c) 2003, Roger Dingledine
 * Copyright (c) 2004-2006, Roger Dingledine, Nick Mathewson.
 * Copyright (c) 2007-2021, The Tor Project, Inc. */
/* See LICENSE for licensing information */

/**
 * \file keyval.h
 *
 * \brief Header for keyval.c
 **/

#ifndef TOR_KEYVAL_H
#define TOR_KEYVAL_H

int string_is_key_value(int severity, const char *string);

#endif /* !defined(TOR_KEYVAL_H) */
