/* Copyright (c) 2001 Matej Pfajfar.
 * Copyright (c) 2001-2004, Roger Dingledine.
 * Copyright (c) 2004-2006, Roger Dingledine, Nick Mathewson.
 * Copyright (c) 2007-2021, The Tor Project, Inc. */
/* See LICENSE for licensing information */

/**
 * @file bridgeauth.h
 * @brief Header for bridgeauth.c
 **/

#ifndef TOR_DIRAUTH_BRIDGEAUTH_H
#define TOR_DIRAUTH_BRIDGEAUTH_H

void bridgeauth_dump_bridge_status_to_file(time_t now);

#endif /* !defined(TOR_DIRAUTH_BRIDGEAUTH_H) */
